//
//  BFRecordScreenCameraManager.swift
//  BFRecordScreenKit
//
//  Created by 胡志强 on 2022/1/13.
//

import Foundation
import GPUImage
import BFCommonKit
import BFUIKit
import BFMediaKit
import CoreMedia
import UIKit


let vpath = recordVideosDirectory + "camera_writer.mov"
let videoSize = CGSize(width: cScreenWidth, height: cScreenWidth * 16 / 9.0)

class BFRecordScreenCameraManager : BFRecordScreenBaseManager{
    //
    var rendView = GPUImageView()
    // 录制完成回调
    var recordEndCallBack: recordEndCallBack?
    var recordProgressCallBack : ((CMTime) -> Void)?
    var startTime = Date()
    let videoModel = PQEditVisionTrackMaterialsModel()
    var firstOpenCamera : Bool = true
    
    fileprivate var timer:Timer?
    // 定时器时间间隔
    var timeDur: TimeInterval = 1

    
    lazy var camera : GPUImageStillCamera? = {
        let camera = GPUImageStillCamera(sessionPreset: AVCaptureSession.Preset.hd1920x1080.rawValue, cameraPosition: AVCaptureDevice.Position.back)
        camera?.outputImageOrientation = UIInterfaceOrientation.portrait
        camera?.addAudioInputsAndOutputs()
        camera?.delegate = self
        return camera
        
    }()
    
    var movieWrite : GPUImageMovieWriter?
    
    override func resetEnv(){
        guard let camera = camera else {
            cShowHUB(superView: nil, msg: "摄像头开启失败！")
            return
        }
        camera.startCapture()

        initerlizeWriter()

        camera.addTarget(filter)
        filter.addTarget(rendView)
        
        if firstOpenCamera {
            startRecord()
            DispatchQueue.main.asyncAfter(deadline: .now() + 0.3, execute: { [weak self] in
                guard let wself = self else { return }
                
                wself.endRecord()
                wself.firstOpenCamera = false
            })
        }
    }
    
    override func startRecord(){
        guard let movieWrite = movieWrite else {
            cShowHUB(superView: nil, msg: "录制启动失败")
            return
        }
        

        super.startRecord()
        if !firstOpenCamera{
            progreddL?.isHidden = false
            progressThumV?.isHidden = false
        }
        
        startTime = Date()
        videoModel.timelineIn = currentAssetProgress.seconds
        
        movieWrite.startRecording()
        
        var thumbCount = -1
        timer = Timer.scheduledTimer(withTimeInterval: 0.01, repeats: true, block: {[weak self] timer in
            guard let wself = self else { return }
            if wself.firstOpenCamera {
                return
            }
            //MARK: 进度计时器
//            let dur = Date().timeIntervalSince(sself.startTime)
            let dur = CMTime(seconds: movieWrite.duration.seconds, preferredTimescale: 1000)
            // 获取缩略图
            let coculationCount = Int(dur.seconds / 5.0)
            if coculationCount > thumbCount{
                BFLog(1, message: "取一张缩略图出来 \(dur.seconds)")
                wself.camera?.capturePhotoAsImageProcessedUp(toFilter: wself.filter, withCompletionHandler: {[weak self] image, err in
                    guard let wself = self else { return }
                    
                    if let img = image{
                        wself.recordItem?.thumbImgs.append(img)
                        wself.updateThumb()
                    }
                })
                thumbCount = coculationCount
            }

            let currDur = CMTime(seconds: wself.videoModel.timelineIn, preferredTimescale: 1000) + dur
            wself.currentAssetProgress = currDur
            wself.recordProgressCallBack?(currDur)
            wself.updateUI(progress: currDur)
//            BFLog(1, message: "currTime \(dur.seconds), \(sself.currentAssetProgress.seconds)")
        })
        RunLoop.current.add(timer!, forMode: .common)
    }
    
    override func endRecord(){
        super.endRecord()
        
        timer?.invalidate()
        timer = nil
        BFLog(1, message: " currTime fini 1 \(Date().timeIntervalSince(startTime))")
        let su = firstOpenCamera
        movieWrite?.finishRecording(completionHandler: {[weak self] in
            guard let sself = self else { return }
            
            if FileManager.default.fileExists(atPath: vpath) {
                if !su {
                    let finalPath = PQBridgeObject.p_setupFileRename(vpath)
                    let dur = AVURLAsset(url: URL(fileURLWithPath: finalPath))
                    BFLog(1, message: "currTime fini 2 \(dur.duration.seconds)")
                    sself.videoModel.timelineOut += dur.duration.seconds
                    sself.videoModel.locationPath = finalPath
                    sself.recordEndCallBack?(true, sself.videoModel)
                }
            }
            
            // 重置writer，提前为下次录制做准备
            sself.initerlizeWriter()
        })
    }
    
    override func clean() {
        super.clean()
    }
    
    // MARK: - 私有方法
    func updateUI(progress:CMTime){
        progreddL?.text = CMTimeGetSeconds(progress).formatDurationToHMS()
    }
    
    func updateThumb(){
        progressThumV!.addThumbImages(images: recordItem!.thumbImgs)
        let size = CGSize(width: recordItem!.materialDuraion.seconds * 70.0 / 5.0, height: progressThumV!.height)
        progressThumV!.progressView.contentSize = size
    }
    
    func beginRecord(startTime:CMTime) {

    }
    
    func initerlizeWriter(){
        if FileManager.default.fileExists(atPath: vpath){
            try? FileManager.default.removeItem(atPath: vpath)
        }
        if movieWrite != nil {
            filter.removeTarget(movieWrite)
//            camera?.audioEncodingTarget = nil
            movieWrite = nil
        }
        
        movieWrite = GPUImageMovieWriter(movieURL: URL(fileURLWithPath: vpath), size: videoSize)

//        movieWrite?.shouldPassthroughAudio = true
//        movieWrite?.hasAudioTrack = true
//        camera?.audioEncodingTarget = movieWrite
        filter.addTarget(movieWrite)

    }
    
}

extension BFRecordScreenCameraManager : GPUImageVideoCameraDelegate {
    func willOutputSampleBuffer(_ sampleBuffer: CMSampleBuffer!) {
        let dur = CMTime(seconds: movieWrite?.duration.seconds ?? 0, preferredTimescale: 1000)
        // 获取缩略图
        let coculationCount = Int(dur.seconds / 5.0)
        if coculationCount >= recordItem?.thumbImgs.count ?? -1{
            BFLog(1, message: "取一张缩略图出来 \(dur.seconds)")
            if let img = sampleBufferToImage(sampleBuffer){
                recordItem?.thumbImgs.append(img)
                updateThumb()
            }
        }
    }
    
    func sampleBufferToImage(_ sampleBuffer: CMSampleBuffer) -> UIImage? {
        guard let imageBuffer = CMSampleBufferGetImageBuffer(sampleBuffer) else {
            return nil
        }

        CVPixelBufferLockBaseAddress(imageBuffer, CVPixelBufferLockFlags(rawValue: 0))

        let baseAddress = CVPixelBufferGetBaseAddress(imageBuffer)
        let bytesPerRow = CVPixelBufferGetBytesPerRow(imageBuffer)
        let width = CVPixelBufferGetWidth(imageBuffer)
        let height = CVPixelBufferGetHeight(imageBuffer)
        let colorSpace = CGColorSpaceCreateDeviceRGB()
        let bitmapInfo = CGBitmapInfo(rawValue: CGImageAlphaInfo.noneSkipFirst.rawValue | CGBitmapInfo.byteOrder32Little.rawValue)

        guard let context = CGContext(data: baseAddress,
                                      width: width,
                                      height: height,
                                      bitsPerComponent:8,
                                      bytesPerRow: bytesPerRow,
                                      space: colorSpace,
                                      bitmapInfo: bitmapInfo.rawValue) else { return nil }

        guard let cgImage = context.makeImage() else {
            return nil
        }

        CVPixelBufferUnlockBaseAddress(imageBuffer, CVPixelBufferLockFlags(rawValue: 0));

        return UIImage(cgImage: cgImage)

    }
}
